/*
 * Copyright (C) 1996-2003 LAAS/CNRS 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
/*
 *
 * Maher Khatib - Tue Jun 10 1997
 *
 */
#ifndef _SICK_LIB_H_
#define _SICK_LIB_H_

/*
 * Sick errors 
 */
#include "sick_lib_error.h"

/*
 * Sick functional modes
 */
#include "sick_lib_modes.h"

/* 
 * Trams description 
 */
#include "sick_lib_trams.h"

/*
 * serial package
 */
#ifndef SICK_SIMUL
#include "serial/serial.h"
#endif

/* cf. doc. SICK-PLS and SICK-LMS */

/* Transmission rate by default */
#define SICK_LIB_DEFAULT_TR_RATE                    (9600)

/* Data tarm length max */
#define SICK_LIB_DATA_LENGTH_MAX                    (0x400)

/* code of no command */
#define SICK_LIB_NO_CMD                             (-1)

/* Max type length */
#define SICK_LIB_TYPE_LENGTH_MAX                    (128)

/* sick scan units */
typedef enum sick_lib_measure_units {
  SICK_LIB_MEASURE_UNIT_CM = 0,
  SICK_LIB_MEASURE_UNIT_MM = 1,
  SICK_LIB_MEASURE_UNIT_10_CM = 2
} SICK_LIB_MEASURE_UNITS;

/* Interpretation des flags de la mesure 
 *
 * Les valeurs > 2 ne sont possibles que pour le firmware LMS >= 1.07 
 * Dans ce cas le nombre de bits significatifs sur la mesure de distance 
 * changent:
 * modes 0, 1, 2: 3 bits de flags + 13 bits de mesure
 * modes 3 et 4: 2 bits de flags + 14 bits de mesure
 * modes 5 et 6: 1 bit de flag + 15 bits de mesure
 *
 * L'interpretation des flags est donnee par le mode.
 * 
 * La portee max du capteur varie selon l'unite de la mesure et le 
 * nombre de bits de la mesure:
 *
 * nombre de bits:  |  13    14     15  |
 * -----------------+-------------------+
 * mm               |   8m   16m    32m |
 * cm               |  80m  160m   320m |
 * 10cm             | 150m   (1)    (1) |
 * -----------------+-------------------+
 * (1) d'apres la doc ces combinaisons ne sont pas defines
 */
typedef enum sick_lib_measure_flags {
  SICK_LIB_MEASURE_FLAGS_ABS = 0,    /* 3 bits: Champ A, Champ B, Saturation */
  SICK_LIB_MEASURE_FLAGS_REF8 = 1,   /* Reflexion sur 3 bits */
  SICK_LIB_MEASURE_FLAGS_ABC = 2,    /* 3 bits: Champ A, Champ B, Champ C */
  SICK_LIB_MEASURE_FLAGS_REF4 = 3,   /* Reflexion sur 2 bits  */
  SICK_LIB_MEASURE_FLAGS_AB = 4,     /* 2 bits: Champ A, Champ B */
  SICK_LIB_MEASURE_FLAGS_REF2 = 5,   /* Reflexion sur 1 bit */
  SICK_LIB_MEASURE_FLAGS_A = 6	     /* 1 bit: Champ A */
} SICK_LIB_MEASURE_FLAGS;
	
/* sick length range  
   (deduced from sick_scanning_resolution and sick_lib_measure_flags) */
typedef enum sick_lib_length_ranges {
  SICK_LIB_LENGTH_RANGE_8_M=8,
  SICK_LIB_LENGTH_RANGE_16_M=16,
  SICK_LIB_LENGTH_RANGE_32_M=32,
  SICK_LIB_LENGTH_RANGE_80_M=80,
  SICK_LIB_LENGTH_RANGE_150_M=150,
  SICK_LIB_LENGTH_RANGE_160_M=160,
  SICK_LIB_LENGTH_RANGE_320_M=320
} SICK_LIB_LENGTH_RANGES;

/* sick scan range */
typedef enum sick_lib_scan_range {
  SICK_LIB_SCAN_RANGE_100 = 100,
  SICK_LIB_SCAN_RANGE_180 = 180
} SICK_LIB_SCAN_RANGE;

/* sick scan resolution */
typedef enum sick_lib_scan_resolution {
  SICK_LIB_SCAN_RESOLUTION_100 = 100,
  SICK_LIB_SCAN_RESOLUTION_50 = 50
} SICK_LIB_SCAN_RESOLUTION;

/*
 * Sick data buffer
 */
typedef struct sick_lib_data_tram_str {
  int len;			   /* length of data */
  char cmd;			   /* command */
  char status;			   /* status of sick */
  char align[2];		   /* alignment */
  char data[SICK_LIB_DATA_LENGTH_MAX];  /* data */
} SICK_LIB_DATA_TRAM_STR;


/* Sick running state */
typedef enum sick_lib_running_state {
  SICK_LIB_OK = 0,
  SICK_LIB_INFO = 1,
  SICK_LIB_WARNING = 2,
  SICK_LIB_ERROR = 3,
  SICK_LIB_FATAL_ERROR = 4
} SICK_LIB_RUNNING_STATE;

/*
 * config str
 */
typedef struct sick_lib_internal_config_str {
  unsigned short min_obj_diameter;	    /* in cm */
  unsigned short stop_thershold;	    /* mv */

  unsigned char fog;			    /* 0 - 1 */
  unsigned char measure_flag;		    /* 0 - 2 */
  unsigned char measure_unit;		    /* SICK_MEASURES_IN_CM, IN_MM, IN_DM */
  unsigned char temp_field;		    /* 0 - 2 */
  unsigned char sub_field;		    /* 0 - 1 */
  unsigned char multiple_eval;		    /* in [ 2 - 125] */
  unsigned char restart;		    /* 0 - 4 */
  unsigned char restart_time;		    /* time in secs */
  unsigned char restart_test;		    /* 0 - 1 -- with - without*/

  unsigned char a_contour_ref_obj_size;	    /* not used */
  unsigned char a_contour_rang_max;	    /*    .     */
  unsigned char a_contour_rang_min;
  unsigned char a_contour_start_angle;
  unsigned char a_contour_stop_angle;

  unsigned char b_contour_ref_obj_size;
  unsigned char b_contour_rang_max;
  unsigned char b_contour_rang_min;
  unsigned char b_contour_start_angle;
  unsigned char b_contour_stop_angle;

  unsigned char c_contour_ref_obj_size;
  unsigned char c_contour_rang_max;
  unsigned char c_contour_rang_min;
  unsigned char c_contour_start_angle;
  unsigned char c_contour_stop_angle;
  
  unsigned char pixel_oriented;		    /* 0 - 1 */
  unsigned char single_measure_mode;	    /* 0 - 1 -- no fade-out - faded-out */
  short zero_point;			    /* measure offset [-32768, 32767] mm */
 /*  short dazzling;   */                         /* !! NEW FIELD for v>=2.0 !!
					       [1-255] default 2 */
} SICK_LIB_INTERNAL_CONFIG_STR;

/*
 * Sick state str
 */
typedef struct sick_lib_state_str {
  int reset;				/* sick was reset */
  int config;				/* TRUE = configurations was read */
  int state;				/* TRUE = global status was read */
  SICK_LIB_RUNNING_STATE run;		/* running status */
  unsigned int start, end;		/* timing */
  char type[SICK_LIB_TYPE_LENGTH_MAX];
} SICK_LIB_STATE_STR;

/*
 * Sick control str
 */
typedef struct sick_lib_control_str {
  int mode;				/* current mode */
  int cmd;				/* current command */
  int result;				/* result of command */
  int baud;				/* current baud rate */
  SICK_LIB_SCAN_RANGE range;		/* current angle range */
  SICK_LIB_SCAN_RESOLUTION resolution;	/* current angle resolution */
  SICK_LIB_MEASURE_UNITS measure_unit;	/* current measures unit */
} SICK_LIB_CONTROL_STR;


/*
 * Sick measures str
 */
typedef struct sick_lib_measures_header_str {
  int n;
  int nothing;
  double theta_min, dtheta;
} SICK_LIB_MEASURES_HEADER_STR;

/*
 * Sick Lib str
 */
typedef struct sick_lib_str {
#ifndef SICK_SIMUL
  SERIAL_BUFFER_STR *serial;			/* serial structure */
#else
  void *serial;
#endif
  SICK_LIB_STATE_STR state;			/* current global state */
  SICK_LIB_DATA_TRAM_STR tram;			/* data tram str */
  SICK_LIB_INTERNAL_CONFIG_STR config;		/* global config */
  SICK_LIB_CONTROL_STR send;			/* control block sent to sick */
  SICK_LIB_CONTROL_STR rcv;			/* control block received from it */
  SICK_LIB_MEASURES_HEADER_STR measures;	/* current measures */
  double x, y, theta;        /* sick position - for simulation only */
} SICK_LIB_STR;

/*
 * Proto
 */
#include "sick_lib_proto.h"

#endif /* _SICK_LIB_H_ */
